﻿using System;
using System.Windows;
using System.Windows.Data;
using System.Windows.Media;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Grid.Demo.Internal;
using Dapfor.Wpf.Utils;

namespace Dapfor.Wpf.Grid.Demo.Examples.DataBinding.ConditionalHierarchicalBinding
{
    /// <summary>
    /// Interaction logic for ConditionalHierarchicalBindingDemo.xaml
    /// </summary>
    [Example("Data binding", "Conditional hierarchical binding", "Description_ConditionalBindingDemo")]
    public partial class ConditionalHierarchicalBindingDemo : IDisposable
    {
        private readonly Collections _collections;
        private readonly Random _r = new Random();
        private readonly Brush _basketColor;
        private readonly Brush _orderColor;
        private readonly Brush _dealColor;

        public ConditionalHierarchicalBindingDemo()
        {
            InitializeComponent();
            _collections = (Collections) FindResource("collections");

            _basketColor = (Brush) FindResource("BasketColor");
            _orderColor = (Brush)FindResource("OrderColor");
            _dealColor = (Brush)FindResource("DealColor");

            grid2.Filter = new Filter(OnRowFilter);

            //Populate with random data
            Populate();
        }

        public void Dispose()
        {
        }

        #region Conditional binding for both grids

        private void grid1_RowAdding(object sender, GridControlRowAddingEventArgs e)
        {
            Basket basket = e.DataObject as Basket;
            if (basket != null)
            {
                e.DataObject = basket.Orders;
                e.InsertionType = InsertionType.AsChildAndCollection;
            }
        }

        private void grid2_RowAdding(object sender, GridControlRowAddingEventArgs e)
        {
            Basket basket = e.DataObject as Basket;
            if (basket != null)
            {
                e.DataObject = basket.Orders;
                e.InsertionType = cbBaskets.IsChecked.Value
                                      ? InsertionType.AsChildAndCollection
                                      : InsertionType.AsCollection;
            }
            Order order = e.DataObject as Order;
            if (order != null)
            {
                e.DataObject = order.Deals;
                e.InsertionType = btnOrders.IsChecked.Value
                                      ? InsertionType.AsChildAndCollection
                                      : InsertionType.AsCollection;
            }
        }

        #endregion

        #region Content filtering

        private bool OnRowFilter(Row row)
        {
            if (cbFilter.IsChecked.Value)
            {
                if (grid1.Selection.Count == 0)
                {
                    return true;
                }

                Basket basket = row.DataObject as Basket;
                if (basket != null)
                {
                    //Check if the basket in the upper grid is selected
                    Row basketRow = grid1.DataObjects.FindFirstRow(basket);

                    //Check if any of basket children in the grid1 is selected
                    bool filtered = basketRow != null && !basketRow.Selected;
                    if (filtered)
                    {
                        foreach (Row child in basketRow.VisibleChildren)
                        {
                            filtered &= !child.Selected;
                            if (!filtered)
                            {
                                break;
                            }
                        }
                    }

                    return filtered;
                }

                Order order = row.DataObject as Order;
                Deal deal = row.DataObject as Deal;

                order = order ?? (deal != null ? deal.Order : null);

                if (order != null)
                {
                    //Check if the basket in the upper grid is selected
                    Row orderRow = grid1.DataObjects.FindFirstRow(order);
                    bool filtered = orderRow != null && !orderRow.Selected;
                    if (filtered && order.Basket != null)
                    {
                        //Check the parent basket
                        Row basketRow = orderRow.Parent;
                        filtered = basketRow != null && !basketRow.Selected;

                    }
                    return filtered;
                }


            }
            return false;
        }

        #endregion

        #region Private methods

        private void Populate()
        {
            for (int i = 0; i < 2; ++i)
            {
                AddBasket();
            }

            for (int i = 0; i < 4; ++i)
            {
                AddOrderToBasket();
            }

            for (int i = 0; i < 2; ++i)
            {
                AddSingleOrder();
            }

            for (int i = 0; i < 15; ++i)
            {
                AddDeal();
            }

        }

        private void AddBasket()
        {
            _collections.Baskets.Add(new Basket("Basket #" + (_collections.Baskets.Count + 1)));
        }

        private void AddOrderToBasket()
        {
            Basket basket = _collections.Baskets.Count > 0 ? _collections.Baskets[_r.Next(_collections.Baskets.Count)] : null;
            if (basket != null)
            {
                Order order = new Order("Order #" + (_collections.AllOrders.Count + 1),
                                        _r.Next() % 2 == 0 ? Way.Buy : Way.Sell,
                                        _r.Next(50, 150) + _r.NextDouble(),
                                        _r.Next(100, 50000),
                                        "Product #" + _r.Next(1, 100),
                                        basket);
                _collections.AllOrders.Add(order);
                basket.AddOrder(order);
            }
        }

        private void AddSingleOrder()
        {
            Order order = new Order("Order #" + (_collections.AllOrders.Count + 1),
                                    _r.Next() % 2 == 0 ? Way.Buy : Way.Sell,
                                    _r.Next(50, 150) + _r.NextDouble(),
                                    _r.Next(100, 50000),
                                    "Product #" + _r.Next(1, 100),
                                    null);
            _collections.AllOrders.Add(order);
            _collections.SingleOrders.Add(order);
        }

        private void AddDeal()
        {
            Order order = _collections.AllOrders.Count > 0 ? _collections.AllOrders[_r.Next(_collections.AllOrders.Count)] : null;
            if (order != null)
            {
                Deal deal = new Deal(order,
                                     "Deal #" + (_collections.AllDeals.Count + 1),
                                     _r.Next(50, 150) + _r.NextDouble(),
                                     _r.Next(50, 1000),
                                     "Counterpart " + _r.Next(1, 100));

                _collections.AllDeals.Add(deal);
                order.AddDeal(deal);
            }
        }

        private void RebindGrid()
        {
            //Rebind the grid to the source
            if (grid2 != null)
            {
                BindingOperations.GetMultiBindingExpression(grid2, GridControl.ItemsSourceProperty).UpdateTarget();

            }
        }

        #endregion

        #region Private event handlers

        private void btnAddBasket_Click(object sender, RoutedEventArgs e)
        {
            AddBasket();
        }
        private void btnAddOrderToBasket_Click(object sender, RoutedEventArgs e)
        {
            AddOrderToBasket();
        }
        private void btnSimpleOrder_Click(object sender, RoutedEventArgs e)
        {
            AddSingleOrder();
        }
        private void btnAddDeal_Click(object sender, RoutedEventArgs e)
        {
            AddDeal();
        }

        private void cbBaskets_CheckChanged(object sender, RoutedEventArgs e)
        {
            RebindGrid();
        }

        private void btnOrders_CheckChanged(object sender, RoutedEventArgs e)
        {
            RebindGrid();
        }

        private void OnRowBackground(object sender, GridControlRowBrushEventArgs e)
        {
            if(e.Row.DataObject is Basket)
            {
                e.Brush = _basketColor;
            }
            if (e.Row.DataObject is Order)
            {
                e.Brush = _orderColor;
            }
            if (e.Row.DataObject is Deal)
            {
                e.Brush = _dealColor;
            }

        }

        private void cbFilter_CheckChanged(object sender, RoutedEventArgs e)
        {
            if (grid2 != null)
            {
                grid2.FilterRefresh();    
            }
        }

        private void grid1_RowSelectionChanged(object sender, GridControlRowEventArgs e)
        {
            grid2.FilterRefresh();
        }

        #endregion
    }
}