﻿using System;
using System.Windows;
using System.Windows.Threading;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Grid.Demo.Internal;
using Dapfor.Wpf.Utils;

namespace Dapfor.Wpf.Grid.Demo.Examples.Performance.RealTimeSorting
{
    /// <summary>
    /// Interaction logic for RealTimeSortingDemo.xaml
    /// </summary>
    [Example("Performance", "Realtime sorting and filtering", "Description_RealTimeSortingDemo")]
    public partial class RealTimeSortingDemo : IDisposable
    {
        private readonly DispatcherTimer _timer = new DispatcherTimer();
        private readonly DataObjectCollection _collection;
        private readonly Random _random = new Random();
        private DateTime _lastTimeStamp = DateTime.MinValue;


        #region Dependency properties

        #region UpdatesCount property

        internal static readonly DependencyProperty UpdatesCountProperty = DependencyProperty.Register("UpdatesCount", typeof(int), typeof(RealTimeSortingDemo), new PropertyMetadata(500));

        internal int UpdatesCount
        {
            get { return (int)GetValue(UpdatesCountProperty); }
            set { SetValue(UpdatesCountProperty, value); }
        }

        #endregion

        #region MinValue property

        internal static readonly DependencyProperty MinValueProperty = DependencyProperty.Register("MinValue", typeof(int), typeof(RealTimeSortingDemo), new PropertyMetadata(100, OnUpdateFilter));

        internal int MinValue
        {
            get { return (int)GetValue(MinValueProperty); }
            set { SetValue(MinValueProperty, value); }
        }

        #endregion

        #region MaxValue

        internal static readonly DependencyProperty MaxValueProperty = DependencyProperty.Register("MaxValue", typeof(int), typeof(RealTimeSortingDemo), new PropertyMetadata(1400, OnUpdateFilter));

        internal int MaxValue
        {
            get { return (int)GetValue(MaxValueProperty); }
            set { SetValue(MaxValueProperty, value); }
        }

        #endregion

        private static void OnUpdateFilter(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var control = (RealTimeSortingDemo) d;
            control.UpdateFilter();
        }

        #endregion



        public RealTimeSortingDemo()
        {
            InitializeComponent();

            _collection = (DataObjectCollection)FindResource("collection");

            grid.Headers[0]["Id2"].SortDirection = SortDirection.Ascending;
            grid.Headers[0]["Id4"].SortDirection = SortDirection.Descending;
            grid.Headers[0]["Id5"].SortDirection = SortDirection.Descending;

            grid.Filter = new Filter(IsFiltered);

            _timer.Interval = TimeSpan.FromMilliseconds(1);
            _timer.Tick += OnTick;
            _timer.Start();
        }

        private void OnTick(object sender, EventArgs e)
        {
            DateTime current = DateTime.UtcNow;
            TimeSpan ts = current - _lastTimeStamp;

            if (_collection != null && _collection.Count > 0 && UpdatesCount > 0)
            {

                int updatesCount = (int)((ts.TotalMilliseconds * UpdatesCount) / 1000);
                updatesCount = Math.Max(1, updatesCount);

                for (int i = 0; i < updatesCount; i++)
                {
                    DataObject obj = _collection[_random.Next(_collection.Count)];
                    obj.RandomUpdate(0, 2000);
                }
            }
            _lastTimeStamp = DateTime.UtcNow;
            tbInfo.Text = String.Format("{0}", grid.Rows.Count);
        }

        public void Dispose()
        {
            _timer.Stop();
            if (_collection != null)
            {
                _collection.Clear();
            }
        }

        void UpdateFilter()
        {
            grid.FilterRefresh();
            grid.Invalidate();
        }

        private bool IsFiltered(Row row)
        {
            //The grid calls this callback in the GUI thread each time when the value of the data object in changed
            foreach (Column column in grid.Headers[0].VisibleColumns)
            {
                int value = (int)row[column.Id].Value;
                //Check for the min bounds
                if (cbMinValue.IsChecked.Value)
                {
                    if (value < MinValue) return true;
                }
                //Check for the max bounds
                if (cbMaxValue.IsChecked.Value)
                {
                    if (value > MaxValue) return true;
                }
            }

            //The row is not filtered
            return false;


        }

        #region Private event handlers

        private void cbMinValue_CheckChanged(object sender, RoutedEventArgs e)
        {
            UpdateFilter();
        }

        private void cbMaxValue_CheckChanged(object sender, RoutedEventArgs e)
        {
            UpdateFilter();
        }

        #endregion
    }
}